@@ -1,15 +1,5 @@
 # Schedule::Load::Hosts.pm -- Loading information about hosts
-######################################################################
-#
-# Copyright 2000-2006 by Wilson Snyder.  This program is free software;
-# you can redistribute it and/or modify it under the terms of either the GNU
-# General Public License or the Perl Artistic License.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
+# See copyright, etc in below POD section.
 ######################################################################
 
 package Schedule::Load::Hosts;
@@ -36,7 +26,7 @@ use Carp;
 # Other configurable settings.
 $Debug = $Schedule::Load::Debug;
 
-$VERSION = '3.060';
+$VERSION = '3.061';
 
 ######################################################################
 #### Globals
@@ -381,6 +371,7 @@ sub print_holds {
 sub print_status {
     my $hosts = shift;
     # Daemon status, mostly for debugging
+    $hosts->_fetch_if_unfetched;
     my $out = "";
     {
 	my @fmts = ("%-^s",    "%^s",     "%-^s",      "%^s",   " %-s");
@@ -879,9 +870,9 @@ The port number of slchoosed.  Defaults to 'slchoosed' looked up via
 
 The latest version is available from CPAN and from L<http://www.veripool.org/>.
 
-Copyright 1998-2006 by Wilson Snyder.  This package is free software; you
+Copyright 1998-2009 by Wilson Snyder.  This package is free software; you
 can redistribute it and/or modify it under the terms of either the GNU
-Lesser General Public License or the Perl Artistic License.
+Lesser General Public License Version 3 or the Perl Artistic License Version 2.0.
 
 =head1 AUTHORS
 
